// HELPER FUNCTIONS
var ww = 96;
ww = 128;
var hh = ww;

var identity_transform = {angle: 0,
			 scale: 1};

function isArray(o) {
    return Object.prototype.toString.call(o) === '[object Array]'; 
}

var identity = function(x){return x;}
var sign = function(x){return x == 0 ? 0 : x > 0 ? 1 : -1;}

var iota = 0.0000001;
function approximately(x,y, tolerance) {
    var tolerance = tolerance || iota;
    return Math.abs(x-y) <= tolerance;}

function distinct(arr) {
    // return a copy of the array with duplicates removed
    var seen = {};
    var ret = [];
    for(var i in arr) {
	if(!seen[arr[i]]){
	    seen[arr[i]] = true;
	    ret.push(arr[i]);
	}
    }
    return ret;
}


function filter(pred, coll) {
    var ret = [];
    for(var i in coll) {
	if(pred(coll[i])){ret.push(coll[i]);}
    }
    return ret;
}

function remove(pred, coll) {
    var ret = [];
    for(var i in coll) {
	if(!pred(coll[i])){ret.push(coll[i]);}
    }
    return ret;
}

function keys(m){var ret = []; for(var x in m){ret.push(x);} return ret;}
function vals(m){var ret = []; for(var x in m){ret.push(m[x]);} return ret;}
function clean(m) {return $.extend(true,{},m);}


function contains(coll, x){
    return (coll.length != 0) && (coll[0] == x || contains(coll.slice(1), x));
}

function determinant(v,w){return v[0]*w[1]-v[1]*w[0];}
function dot_product(v,w){return v[0]*w[0]+v[1]*w[1];}
function cross_product(v,w) {return v[0]*w[1] - v[1]*w[0];} //signed magnitude of the cross product

function orient_counterclockwise(scc) {
    // Modifies the representation of the simple closed curve so that
    // its vertices are listed in counterclockwise order.

    var v = vector_subtract(scc.junction[0],scc.junction[2]);
    var w = vector_subtract(scc.junction[4],scc.junction[2]);
    
    if(cross_product(v,w) > 0) {
	var ret = scc.map(null, function invert(x){return -x;});
	ret.junction = ret.junction.reverse();
	return ret;
    }
    return scc;
}



// ********* GEOMETRIC FEATURES AND RELATIONSHIPS
function arc_length(point_A, curvature, point_B) {
    var d = distance(point_A, point_B);
    if(curvature == 0){return d;}
    return 2/curvature*Math.asin(
	d*curvature/2
    );
}
function arc_centroid(point_A, curvature, point_B) {
    // return the centroid of the given arc
    // (signed curvature indicates direction, by convention)
    
    var midpoint = vector_scale(0.5, vector_add(point_A, point_B)); 
    if(curvature == 0) {
	return midpoint;
    }
    var d = distance(point_A, point_B);
    var angle = Math.asin(d*curvature/2);
    // HALF the angle contained between point_A and point _B (& center of arc's circle)

    // Given a circular segment centered at the origin
    // and oriented symmetrically about the y-axis, its 
    // x-center is 0 and its y-center is as follows:
    var y_center = 4*Math.pow(Math.sin(angle),3)/(3*curvature*(2*angle - Math.sin(2*angle)));

    
    // Translate to get the distance from the midpoint to the
    // centroid.
    y_center = Math.abs(y_center) - sign(curvature)*Math.cos(angle)/curvature;
   
    
    // Get a vector pointing from the midpoint to the tip of the arc.
    var perp = (function(x) {
	return vector_scale(sign(curvature), [x[1], -x[0]]);
    })(vector_subtract(point_B, point_A));


    if(curvature < 0) {
    console.log("norm perp",normalize(perp));
    console.log("y_center",y_center);
    console.log("midpoint",midpoint);
    console.log("scaled perp",vector_scale(y_center, normalize(perp)));
    console.log("final",vector_add(midpoint, vector_scale(y_center, normalize(perp))));
    }

    // Scale the vector accordingly, and add it to the midpoint to get the final result.
    return vector_add(midpoint,
		      vector_scale(y_center,
				   normalize(perp)));

    
}
function centroid(fig) {
    // Approximate the geometric center of the figure using the center
    // of mass of its junctions.
    if(fig.type == "DOT") {
	return fig.position;
    }
    else if(fig.type == "SCC") {
	var centroid = [0,0];
	var total_arc_length = 0;
	for(var i=0; i<fig.junction.length-1; i+=2) {
	    var a = fig.junction[i+0];
	    var b = fig.junction[i+1];
	    var c = fig.junction[i+2];
	    

	    var ds = arc_length(a,b,c);
	    total_arc_length += ds;	    
	    
	    centroid = vector_add(centroid,
				  vector_scale(ds, arc_centroid(a,b,c)));
	}
	return vector_scale(1/total_arc_length, centroid);

	var pts = fig.points();
	return vector_scale(1/pts.length, pts.reduce(vector_add));
    }
    else {
	var pts = [];
	for(var x in fig.coordinates){pts.push(fig.coordinates[x]);}
 
	var curves = fig.curves();
	for(var i in curves) {
	    for(var j=2;j<curves[i].length-2;j+=2) {
		pts.push(curves[i][j]);
	    }
	}
	
	pts = distinct(pts);
	return vector_scale(1/pts.length, pts.reduce(vector_add));
    }
}

function center(fig) {
    // Approximate the geometric center of the figure using the center
    // of mass of its junctions.
    if(fig.type == "DOT") {
	return fig.position;
    }
    else if(fig.type == "SCC") {
	var pts = fig.points();
	return vector_scale(1/pts.length, pts.reduce(vector_add));
    }
    else {
	var pts = [];
	for(var x in fig.coordinates){pts.push(fig.coordinates[x]);}
 
	var curves = fig.curves();
	for(var i in curves) {
	    for(var j=2;j<curves[i].length-2;j+=2) {
		pts.push(curves[i][j]);
	    }
	}
	
	pts = distinct(pts);
	return vector_scale(1/pts.length, pts.reduce(vector_add));
    }
}

function square(x) {return Math.pow(x,2);}

function vector_add(v1, v2) {
    return [v1[0] + v2[0],
	    v1[1] + v2[1]];
}
function vector_subtract(v1, v2) {
    return [v1[0] - v2[0],
	    v1[1] - v2[1]];}

function vector_scale(a, v) {
    var w = [];
    for(var i in v) {w[i] = a * v[i];}
    return w;
}



function normalize(v) {
    var len = distance([0,0],v);
    if(len == 0){return v}
    return vector_scale(1/len, v);
}

function vector_rotate(angle, v) {
    return [v[0] * Math.cos(angle) - v[1] * Math.sin(angle),
	    v[0] * Math.sin(angle) + v[1] * Math.cos(angle)
    ];
}

function distance(point_A, point_B) {
    return Math.sqrt(
	square(point_A[0] - point_B[0]) 
	+ square(point_A[1] - point_B[1]));
}



function compass_relation(fig1, fig2) {
    // Returns a triple representing the relationship between figures
    // 1 and 2.  The first entry is either ABOVE or LEFT, and the next
    // two entries are fig1, fig2 in the appropriate order.

    // Note: added fourth entry to 'triple' indicating whether fig 1
    // comes first (+1) or fig2 (-1).

    var center1 = centroid(fig1);
    var center2 = centroid(fig2);

    var diff = vector_subtract(center2, center1);
    var a = diff[1] > diff[0];
    var b = diff[1] > -diff[0];

    if(a && b) {
	return ["ABOVE", fig2, fig1, -1];
    }
    if(a && !b){
	return ["LEFT", fig2, fig1, -1];
    }
    if(!a && b) {
	return ["LEFT", fig1, fig2, 1];
    }
    if(!a && !b) {
	return ["ABOVE", fig1, fig2, 1];
    }
    
}




// ************* ABSTRACT SHAPE AND DIAGRAM CREATION
// todo: when drawing, return the svg object. 


function dot(position) {
    var ret = {
	type: "DOT",
	position : position,
	draw : function($svg, attrs) {
	    var attrs = $.extend({fill:"#000"}, attrs||{});
	    var xy = to_page($svg,position); 
	    $svg.circle(xy[0],xy[1], to_page($svg, 0.015), attrs);
	}
    };

    ret.map = function(f,g) {
	// apply f to all the points, and g to all the curvatures.
	f = f || identity;
	var clone = $.extend(true, {}, this);
	clone.position = f(this.position);
	return clone;
    };

    return ret;
}
function simple_closed_curve() {
    // todo: ensure the clockwise points convention
    var junction = [];
    for(var i=0; i<arguments.length; i++){junction.push(arguments[i])}
    // todo: input format checking
    junction.push(junction[0]);

    var ret = {
	type: "SCC",
	junction : junction,
	draw : function($svg, attrs) {
	    var attrs = attrs || {};
	    for(var i=0; i<this.junction.length-1; 
		i += 2) {
		draw_arc($svg, 
			 this.junction[i+0],
			 this.junction[i+1],
			 this.junction[i+2],
			attrs);
	    }
	}
    };

    ret.points = function() {
	var pts = [];
	for(var i=0; i< this.junction.length-1; i+=2){
	    pts.push(this.junction[i]);
	}
	return pts;
    };
    ret.curvatures = function() {
	var ks = [];
	for(var i=1; i< this.junction.length-1; i+=2){
	    ks.push(this.junction[i]);
	}
	return ks;
    };


     ret.map = function(f,g) {
	 // apply f to all the points, and g to all the curvatures.
	 f = f || identity;
	 g = g || identity;
	 var clone = $.extend(true, {}, this);
	 for(var i in this.junction) {
	     clone.junction[i] = (i%2==0) ? 
		 f(this.junction[i]) :
		 g(this.junction[i]);
	 }
	 return clone;
    };


    ret.transform = function (T) {
	// Transform the figure, keeping its center of mass fixed
	var T = $.extend({angle:0, scale:1}, T);

	var center = centroid(this);
	return this.map(
	    function(pt) {
		var qt = pt;
		qt = vector_subtract(qt,center);
		qt = vector_rotate(T.angle, qt);
		qt = vector_scale(T.scale, qt);
		qt = vector_add(qt, center);
		return qt;
	    },
	    function(k) {
		return k/T.scale;
	    }
	);
    };
    


    return orient_counterclockwise(ret);
}

function flip_connection(cxn) {
    // reverse the list and invert the curvatures
    var ret = cxn.slice(0).reverse();
    for(var i=0; i<ret.length;i+=2) {
	ret[i] = ret[i] * -1;
    }
    return ret;
}

function revolve_curve(curve) {
    // Reorder the vertexes of the simple closed curve by putting the
    // first one at the end. This changes the representation, but not
    // the shape itself.

    
    var junct = curve.junction;
    junct = junct.slice(2,-1).concat(junct.slice(0,2)).concat([junct[2]]);
    
    return $.extend(curve,{junction: junct});

}

/////////

function connected_figure(labels) {
    var ret = {
	type: "FIG",
	coordinates: labels || {},
	junctions: {}
    };
    ret.add_point = function(label, coordinates) {
	this.coordinates[label] = coordinates;
	return this;
    };
    ret.add_connection = 
	function() {
	    var cxns = {};
	    var args = [];
	    for(var i=0;i<arguments.length;i++) {args.push(arguments[i]);}
	    
	    for(var i=0; i<args.length-1; i+=3) {
		var p = args[i+0];
		var q = args[i+1];
		var r = args[i+2];

		cxns[p] = cxns[p] || [];
		cxns[q] = cxns[q] || [];

		cxns[p].push([q,r]);
		cxns[q].push([p,flip_connection(r)]);
	    }

	    this.junctions = $.extend(this.junctions, cxns);
	    
	    return this;
	};
    
    ret.curves = function(label_start, label_end) {
	// Return all the curves in the figure as alternating
	// coordinate/curvature lists.  If labels are provided,
	// includes only those curves that start at the first and end
	// at the second.

	var curves = []; 
	for(var start in this.junctions) {
	    for(var i in this.junctions[start]) {
		var end = this.junctions[start][i][0];
		var curve = this.junctions[start][i][1].slice(0);
		curve.unshift(this.coordinates[start]);
		curve.push(this.coordinates[end]);
		
		if((!label_start ||  start == label_start) &&
		   (!label_end || end == label_end)) {
		    curves.push(curve);
		}
	    }
	}
	return curves;
    };

    ret.draw = function($svg, attrs) {
	var attrs = attrs || {};
	var curves = this.curves();
	for(var i in curves) {
	    for(var j=0; j<curves[i].length-1; j+=2) {
		if(true || curves[i][j+0] < curves[i][j+2]) {
		draw_arc($svg, 
		    	 curves[i][j+0],
		    	 curves[i][j+1],
		    	 curves[i][j+2],
			attrs);
		}
	    }
	}
	return this;
    };


    ret.map = function(f,g) {
	 // apply f to all the points, and g to all the curvatures.
	f = f || identity;
	g = g || identity;
	var clone = $.extend(true, {},this);
	for(var x in this.coordinates) {
	    clone.coordinates[x] = f(this.coordinates[x]);
	}
	for(var start in this.junctions) {
	    for(var i in this.junctions[start]) {
		for(var j in this.junctions[start][i][1]) {
		    clone.junctions[start][i][1][j] = (j%2==1) ? 
			f(this.junctions[start][i][1][j]) :
			g(this.junctions[start][i][1][j]);
		}
	    }
	}
	return clone;
    };
    

    return ret;
}



function diagram() {
    var ret = {figures : {}};

    ret.add_figure = function(fig) {
	return this;
    };
}




// ********************* TRANSFORMATIONS & SIMILARITY

// mirrors

function compatible_maps(map1, map2) {
    // Return true if identical keys map to the identical values,
    // and if identical values are mapped to by identical keys.

    for(var k in map1) {
	if(map2[k] && map2[k] != map1[k]){return false;}
    }

    val:
    for(var i in vals(map1)) {
	var v = vals(map1)[i];
	for(var k in map2) {
	    if(map2[k] == v) {
		if(map1[k] == v) {
		    continue val;
		}
		else {
		    return false;
		}
	    }
	}
    }
    return true;
}

function same_transform(T1, T2) {
    T1.angle = T1.angle || 0;
    T2.angle = T2.angle || 0;
    T1.scale = T1.scale || 1;
    T2.scale = T2.scale || 1;
    T1.align = T1.align || {};
    T2.align = T2.align || {};
    
    
    for(x in T2.align) {

	if(T1.align[x] && (T1.align[x] != T2.align[x])) {
	    return false;
	}
    }
    
    return approximately(T1.angle, T2.angle) &&
	approximately(T1.scale, T2.scale);

}



function same_point(p, q) {
    // Given two coordinate pairs, return true if the coordinates are
    // approximately equal, otherwise false.
    return approximately(p[0], q[0]) && approximately(p[1], q[1]);
}
function same_junction(p,q) {
    // Given two alternating point/curvature lists, return true if
    // they represent the same junction.
    if(p.length != q.length){return false;}
    for(var i in p) {
	if( (i%2) == 0 && !same_point(p[i], q[i])) {
	    return false;
	}
	if( (i%2) == 1 && !approximately(p[i], q[i])) {
	    return false;
	}
    }
    return true;
}

function identical(fig_1, fig_2) {
    // Return true if and only if fig_1 and fig_2 are exactly the same
    // representations of the same shape in the same position, etc.
    if(fig_1.type != fig_2.type){return false;}
    
    if(fig_1.type == "DOT") {
	return same_point(fig_1.position, fig_2.position);
    }
    else if (fig_1.type = "SCC") {
	if(fig_1.junction.length != fig_2.junction.length) {return false;}
	return same_junction(fig_1.junction, fig_2.junction);
    }
    else {
	//TODO: identical for general connected figures
	
    }
    
}


function align_segment( start_1, end_1, 
			     start_2, end_2) {
    // Given the endpoints of two line segments, return the scale-then-rotate
    // transform that moves the first onto the second.
    
    var point_A = vector_subtract(start_1, end_1);
    var point_B = vector_subtract(start_2, end_2);    
    
    var scale = 
	distance([0,0], point_B) / 
	distance([0,0], point_A);

    var a = distance([0,0], point_A);
    var b = distance([0,0], point_B);
    var c = distance(point_A, point_B);

    //determinant(point_A,point_B)
    var angle = 1*Math.acos((a*a + b*b - c*c)/(2*a*b));

    return {
	scale : scale,
	angle:angle
    };
}




function align_simple_closed_curve(curve_1, curve_2) {
    // Return a list of scale-then-rotate transformations that move
    // curve_1 onto curve_2 (or null, if none exists.)

    // TODO: Ensure the curves are both oriented counterclockwise

    var ps1 = curve_1.points();
    var ps2 = curve_2.points();
    if(ps1.length != ps2.length){return null;}
    
    var qs1 = curve_1.curvatures();
    var qs2 = curve_2.curvatures();

    // Iteratively try to align the first point of curve_1 with each
    // point of curve_2 in turn; return a list of the transformations
    // corresponding to successful alignments.

    
    for(var i in ps2) {
	var T = align_segment(ps1[0],
			      ps1[1],
			      ps2[i],
			      ps2[(i+1)%ps2.length]);
	var candidate = curve_1.transform(T);
	var diff = vector_subtract(curve_2.points()[i],
				   candidate.points()[0]);
	candidate = candidate.map(
	    function(pt) {
		return vector_subtract(pt, diff);
	    },
	    null
	);
	
    }


}




function align_junction(fig_1, fig_2, label_1, label_2, align_default) {
    // Given two general connected figures and a junction from each,
    // return a list of scale-then-rotate transformations that move
    // the first onto the second (or null if none exists).

    // TODO: Check curvature, rotational polarity

    var ret = [];

    if( fig_1.curves(label_1).length !=
	fig_2.curves(label_2).length) {
	return null;
    }
    
    var align = align_default || {};
    if(align[label_1] && align[label_1] != label_2){return null;}
    
    align[label_1] = label_2;

    var pt_1 = fig_1.coordinates[label_1];
    var pt_2 = fig_2.coordinates[label_2];


    // list of the labels of neighboring vertexes (no repeats).
    var neighbors_1 = distinct(fig_1.junctions[label_1].map(
	function(x){return x[0]}));
    var neighbors_2 = distinct(fig_2.junctions[label_2].map(
	function(x){return x[0]}));


    if(neighbors_1.length != neighbors_2.length){return null;}

    var solve_constraint = function(stack) {

	if(stack.length == 0){
	    return; // no more viable options
	}

	var csp = stack.shift();

	if(csp.transform != null) {
	    // The transform has already been decided, so remove all
	    // aligned variables from the "open list".
	    
	    var a = $.extend(true, {},csp.align);	    
	    csp.neighbors_1 = remove(
		function(x){
		    return contains(keys(a), x);
		},
		csp.neighbors_1);
	    csp.neighbors_2 = remove(
		function(x){
		    return contains(vals(a), x);
		},
		csp.neighbors_2);
	}

	
	
	if(csp.neighbors_1.length == 0) {
	    // No more assignments to be made
	    csp.transform["align"] = csp.align;
	    ret.push(csp.transform);
	    return;
	}

	var label_3 = csp.neighbors_1[0];
	var pt_3 = fig_1.coordinates[label_3];

	for(var i=0; i< csp.neighbors_2.length; i++) {
	    var label_4 = csp.neighbors_2[i];
	    if(csp.align[label_3] && csp.align[label_3] != label_4) {
		continue;
	    }
	    
	    
	    var pt_4 = fig_2.coordinates[label_4];
	    

	    // TODO: align the entire family of curves between the two pairs of labelled points.
	    var transform = align_segment(pt_1, pt_3,
					   pt_2, pt_4);

	    if(csp.transform == null) {
		var temp_csp = $.extend(true, {}, csp);
		

		temp_csp.align[label_3] = label_4;
		temp_csp.neighbors_1 = [];
		temp_csp.neighbors_2 = [];
	

		for(var j in csp.neighbors_1) {
		    if(!temp_csp.align[csp.neighbors_1[j]]) {
			temp_csp.neighbors_1.push(csp.neighbors_1[j]);
		    }
		}
		for(var j in csp.neighbors_2) {
		    if(!contains(vals(align), csp.neighbors_2[j])) {
			temp_csp.neighbors_2.push(csp.neighbors_2[j]);
		    }
		}
	
		temp_csp.transform = $.extend(true, {},transform);
		stack.push(temp_csp);
	    }
	    else {
		// Ensure that the assignment is compatible with the
		// existing transform.

		if(same_transform(transform, csp.transform)) {
		    var temp_csp = $.extend(true, {}, csp);
		    temp_csp.align[label_3] = label_4;
		    // remove label_3 and label_4 from consideration
		    temp_csp.neighbors_1.slice(0,1);
		    temp_csp.neighbors_2.slice(i,1);
		    //console.log("SAME TRANSFORM", temp_csp);
		    stack.push(temp_csp);
		}

	    }
	}
	solve_constraint(stack);


    };

    
    solve_constraint(
	[{transform: null,
	  align : align,
	  neighbors_1 : neighbors_1,
	  neighbors_2 : neighbors_2}]
    );

    return ret.length == 0 ? null : ret;
}

function align_connected(fig_1, fig_2) {
    // Given two general connected figures, return a list of
    // transformations sending the first to the second (or null, if
    // none exist).

    // To do: constraint satisfaction problem, where the object is to
    // associate each of the vertexes in fig1 with fig2 under the same
    // transformation.

    // 0. If there are no more vertexes to be considered, return all
    // existing transformations/associations.

    // 1. Otherwise, take the first vertex and its
    // neighbors. Iteratively consider every compatible vertex from
    // the second figure (i.e. figures with the same neighbor count).

    // 2. If no assignments have been made yet, there will be no
    // candidate transformations yet; therefore, set the candidate
    // transforms to be the set of all possible transforms that align
    // this vertex with some vertex from the other figure. Otherwise,
    // reject all vertexes from the other figure that can't be aligned
    // under any candidate transform. (This is a sub-csp, where the
    // goal is to just align /these/ vertexes so that a transformation
    // applies to them.)

    var ret = [];
    var transforms = [];
    var solve_constraint = function(stack) {
	
	if(stack.length == 0){return;}
	var csp = stack.shift();

	if(keys(csp.align).length > 0) {
	    csp.vertexes_1 = remove(
		function(x){return contains(keys(csp.align), x);},
		csp.vertexes_1);
	    csp.vertexes_2 = remove(
		function(x){return contains(vals(csp.align), x);},
		csp.vertexes_2);
	}


	if(csp.vertexes_1.length == 0) {
	    delete csp.transform.align;
	    ret.push({transform: csp.transform, 
		     align: csp.align});
	    return;
	}

	for(var i=0; i<csp.vertexes_1.length; i++) {    
	    for(var j=0; j<csp.vertexes_2.length; j++) {
		var transforms = align_junction(fig_1, fig_2, csp.vertexes_1[i], csp.vertexes_2[j]);
		if(transforms == null){continue;}
		for(var k=0; k<transforms.length;k++) {
		    if(csp.transform && !same_transform(csp.transform, transforms[k])){
			continue;
		    }
		    if(!compatible_maps(transforms[k].align, csp.align)){
			continue;
		    }

		    var temp_csp = $.extend(true, {}, csp);
		    temp_csp.align = $.extend(true, temp_csp.align, transforms[k].align);
		    temp_csp.transform = transforms[k];
		    stack.push(temp_csp);
		}


	    }
	}
	solve_constraint(stack);
	


    };

    solve_constraint([{
	transform: null,
	align: {},
	vertexes_1 : keys(fig_1.coordinates),
	vertexes_2 : keys(fig_2.coordinates),
    }]);
    return ret;
}






// ************** DRAWING FUNCTIONS (low-level)
var pen_style = {
    fill:'none',
    stroke: '#444444',
    strokeWidth:1.6,
};


function svg_new(width, height) {
    var svg = document.createElementNS("http://www.w3.org/2000/svg", "svg");
    svg.setAttribute('width', width);
    svg.setAttribute('height', height);
    svg.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:xlink", "http://www.w3.org/1999/xlink");
    return svg;
}

function to_page($svg, xy) {
    // Convert Evans's unit square coordinates to svg coordinates.
    var w = ww;
    var h = w;

    if(typeof xy == "number"){
	return xy * w;
    }
    else {
	return [xy[0] * w, (1-xy[1]) * h];
    }
}

function draw_arc($svg, start, curve, end, attrs) {
    var attrs = $.extend({},pen_style,attrs);
    var start = to_page($svg, start);
    var end = to_page($svg, end);


    var path = $svg.createPath();
    path.move(start[0],start[1]);
    if(curve == 0) {
	path.line(end[0],end[1]);
    }
    else {
	
	var radius = to_page($svg, Math.abs(1/curve));

	path.arc(
	    radius,
	    radius, 
	    0, 
	    false, 
	    !(curve > 0), 
	    end[0], 
	    end[1]);
    }
    $svg.path(path,attrs);
	
}






// ********************** DEMONSTRATIONS
function fig_5() {
    // Shows how to construct simple closed curves.
    var figs = [
	simple_closed_curve(
	    [0.2, 0.3], 0, 
	    [0.8, 0.3], 0,
	    [0.8, 0.7], 0, // <<<
	    [0.2, 0.7], 0
	),
	simple_closed_curve(
	    [0.2, 0.3], 0, 
	    [0.8, 0.3], 0,
	    [0.8, 0.7], 1, // <<<
	    [0.2, 0.7], 0
	),
	simple_closed_curve(
	    [0.2, 0.3], 0, 
	    [0.8, 0.3], 0,
	    [0.8, 0.7], -1, // <<<
	    [0.2, 0.7], 0
	),
	simple_closed_curve(
	    [0.5, 0.7], 5, 
	    [0.5, 0.3], 5,
	    [0.7, 0.5], 0
	)
    ];

 
    $(figs).each(function(_, fig) {
	var svg = svg_new(ww,hh);
	
	document.body.appendChild(svg);
	var $svg = $(svg).last().svg().svg("get");
	fig.draw($svg);
	for(var i=0; i<fig.junction.length-1;i+=2) {
	    // dot(arc_centroid(fig.junction[i+0],
	    // 		     fig.junction[i+1],
	    // 		     fig.junction[i+2])).draw($svg);
	}
	//dot(centroid(fig)).draw($svg);
    });

}




// ********************* INITIALIZATION

$('body').ready(function() {
   // var $svg = $("#paper").svg().svg("get"); 
    var $svg;

    // Draw a few sample figures
    //fig_5();

    var letter_A = connected_figure(
	{"A" : [0.3, 0.3], //V1
	 "B" : [0.4, 0.5], //V2
	 "C" : [0.6, 0.5], //V3
	 "D" : [0.7, 0.3], //V4
	}
    ).add_connection(
	"A","B", [0],
	"B", "C", [0],
	"B", "C", [0, [0.5, 0.7], 0],
	"C", "D", [0]
    );

    var letter_A_sideways = connected_figure(
   	{"V5" : [0.4, 0.6],
   	 "V6" : [0.5, 0.45],
   	 "V7" : [0.4, 0.4],
   	 "V8" : [0.5, 0.55],
	    
   	}
    ).add_connection(
   	"V5", "V8", [0],
   	"V6", "V7", [0],
   	"V6", "V8", [0, [0.6, 0.5], 0],
   	"V6", "V8", [0]
	
    );


    var fan = connected_figure(
   	{"A":[0.5, 0.2],
   	 "B":[0.5, 0.5]
   	}
    ).add_connection(
   	"A","B",[0],
   	"B","B",[0, [0.7, 0.7], 2.5, [0.3, 0.7], 0]
    );


    var svg = svg_new(ww,hh);
    document.body.appendChild(svg);
    var $svg = $(svg).last().svg().svg("get");


    letter_A_sideways.draw($svg);

    var a = center(simple_closed_curve(
   	    [0.5, 0.7], 5, 
   	    [0.5, 0.3], 5,
   	    [0.7, 0.5], 0
   	));
    var a = center(letter_A); 

    //console.log(align_connected(letter_A, letter_A_sideways));
    
    

    var rect = simple_closed_curve(
   	[0.5, 0.7], 5, 
   	[0.5, 0.3], 5,

   	[0.7, 0.5], 0
   	);


    // var rect2 = simple_closed_curve(
    // 	[0.5, 0.3], 0, 
    // 	[0.5, 0.5], 5,
    // 	[0.7, 0.7], 5
    // );

    var tri = simple_closed_curve(
	[0.5,0.865], 0,
	[0.4,0.7], 0,
	[0.6,0.7], 0

    );
   
    tri.draw($svg);
    
    console.log(compass_relation(letter_A_sideways, tri));

    if(false) {
	rect.draw($svg);
    }
    //dot(centroid(rect)).draw($svg);
    
    //rect.transform({angle:Math.PI/2}).draw($svg);    
  
    //align_simple_closed_curve(rect, rect2);

  
});
